import pygtk; pygtk.require("2.0")
import gtk
import gconf
import gnome.ui
import os

GNOME_APP_ID = "gtstarter"
SCRIPT_VERSION = "0.4"
SCRIPT_NAME = "GTStarter"

SCRIPT_COPYRIGHT = "(c) 2004 Franz Klammer"
SCRIPT_DESCR = "Start the gnome-terminal with a configured profile"
SCRIPT_AUTHORS = ["Franz Klammer <klammer@webonaut.com>"]
SCRIPT_DOCU = []
SCRIPT_TRANS = "TODO..."
GTS_ICON = "/usr/X11R6/share/gnome/pixmaps/nohost.png"
GTS_ABOUT_PIX = "/usr/X11R6/share/gnome/pixmaps/nohost.png"

class gt_starter(gnome.ui.App):
	
	def __init__(self):
		gnome.program_init(GNOME_APP_ID, SCRIPT_VERSION)
		gnome.ui.App.__init__(self, GNOME_APP_ID, SCRIPT_NAME)
  
		self.__gt_gconf_globals = "/apps/gnome-terminal/global/profile_list"
		self.__icon_default_path = "/usr/X11R6/share/gnome/pixmaps/"
		self.__gconf_client = gconf.client_get_default()
		
		self.__start_double_click = False
		self.__keep_open = False				# don't close app after terminal start

		self.__my_conf_path = "/apps/"+GNOME_APP_ID
		self.__gconf_client.add_dir(self.__my_conf_path, gconf.CLIENT_PRELOAD_NONE)

		self.__cursor = "out"
		
		self.__active_profile = 0
		
		self.__context_menu = None
		
		
	def __load_session_list(self):
		self.__profile_list = self.__gconf_client.get_list(self.__gt_gconf_globals, gconf.VALUE_STRING)


	def __load_config(self):
		self.__start_double_click = self.__gconf_client.get_bool(self.__my_conf_path+"/double_click")
		self.__double_click_menu_item.set_active(self.__start_double_click)
		self.__keep_open = self.__gconf_client.get_bool(self.__my_conf_path+"/keep_open")
		self.__keep_open_menu_tiem.set_active(self.__keep_open)


	def __set_click_behaviour(self, menu_nr, menu_item_obj):
		self.__start_double_click = menu_item_obj.active
		self.__gconf_client.set_bool(self.__my_conf_path+"/double_click", self.__start_double_click)


	def __set_quit_behaviour(self, menu_nr, menu_item_obj):
		self.__keep_open = menu_item_obj.active
		self.__gconf_client.set_bool(self.__my_conf_path+"/keep_open", self.__keep_open)
				
		
	def __get_menubar(self):
		menu_items = (
			( "/_File",			None, None, 				0, "<Branch>" ),
			( "/File/_Quit",		None, gtk.main_quit, 			0, "<StockItem>", gtk.STOCK_QUIT ),
			( "/_View",			None, None, 				0, "<Branch>"),
			( "/View/_Refresh", 		None, self.__rebuild_iconlist,		0, "<StockItem>", gtk.STOCK_REFRESH),
			( "/_Settings",			None, None, 				0, "<Branch>" ),
			( "/Settings/ double click",	None, self.__set_click_behaviour,	0, "<ToggleItem>"),
			( "/Settings/ keep open",	None, self.__set_quit_behaviour,	0, "<ToggleItem>"),
			( "/_Help",			None, None, 				0, "<LastBranch>" ),
			( "/Help/_About",		None, self.__show_about, 		0, "<StockItem>", "gnome-stock-about"),
			)


		accel_group = gtk.AccelGroup()
		item_factory = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)

		item_factory.create_items(menu_items)

		self.__double_click_menu_item = item_factory.get_item("/Settings/ double click")
		self.__keep_open_menu_tiem = item_factory.get_item("/Settings/ keep open")

		self.add_accel_group(accel_group)
		menu = item_factory.get_widget("<main>")
		menu.show()

		return menu


	def __get_toolbar(self):
		toolbar = gtk.Toolbar()
		toolbar.show()
		tb1 = toolbar.insert_stock(gtk.STOCK_QUIT, "Quit", "", gtk.main_quit, 0, -1)
		tb2 = toolbar.insert_stock(gtk.STOCK_REFRESH, "Refresh", "", self.__rebuild_iconlist, 0, -1)
		return toolbar


	def __rebuild_iconlist(self, *argv):
		self.__build_iconlist()


	def build_gui(self):
		self.set_menus(self.__get_menubar())
		self.__toolbar = self.__get_toolbar()
		self.set_toolbar(self.__toolbar)

		self.connect('destroy', gtk.main_quit)
		self.set_default_size(420, 320)
		self.set_position(gtk.WIN_POS_CENTER)
		self.set_icon(gtk.gdk.pixbuf_new_from_file(GTS_ICON))
		self.__statusbar = gtk.Statusbar()
		self.__statusbar.set_has_resize_grip(gtk.TRUE)
		self.__statusbar.show()

		self.__iconlist_adjustment = gtk.Adjustment()
		self.__iconlist = gnome.ui.IconList(100, self.__iconlist_adjustment, 0)
		self.__iconlist.connect("select_icon", self.__select_handler)
		self.__iconlist.connect("motion-notify-event", self.__mouse_motion_handler)
		self.__iconlist.show()
		
		self.__build_iconlist()
		
		self.__scrolledwin = gtk.ScrolledWindow()
		self.__scrolledwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.__scrolledwin.show()
		self.__scrolledwin.add(self.__iconlist)

		self.set_statusbar(self.__statusbar)
		self.set_contents(self.__scrolledwin)
		

		self.__load_config()

		self.show()
		gtk.main()
	

	def __build_iconlist(self):

		self.__load_session_list()
		self.__iconlist.clear()
		
		for profile_name in self.__profile_list:
			profile_icon_name = self.__get_icon_from_profile(profile_name)
			profile_icon = gtk.gdk.pixbuf_new_from_file(profile_icon_name)
			icon_name = gconf.unescape_key(profile_name, len(profile_name))
			new_ico = self.__iconlist.append_pixbuf(profile_icon, icon_name, icon_name)

		
	def __select_handler(self, iconlist_obj, icon_idx, gdk_event_object):

		if gdk_event_object:
			#print gdk_event_object.type, gdk_event_object.button

			if  self.__start_double_click:
				if gdk_event_object.type == 5 and gdk_event_object.button == 1:
					self.__starter()
			else:
				if gdk_event_object.type == 4 and gdk_event_object.button == 1:
					self.__starter()

			if gdk_event_object.type == 4 and gdk_event_object.button == 3:
				self.__right_click_menue(icon_idx, gdk_event_object)

				
	def __right_click_menue(self, icon_idx, gdk_event_object):
		
		if not self.__context_menu:
			entries = [
				(gtk.STOCK_OPEN, self.__start_with_window, 1)			
			]
			#(gtk.STOCK_PROPERTIES, self.__change_profile_icon, 1)
	
			self.__context_menu = gtk.Menu()
			for stock_id,callback,sensitivity in entries:
				item = gtk.ImageMenuItem(stock_id)
				
				if callback:
					item.connect("activate",callback)
					
				item.set_sensitive(sensitivity)
				item.show()
				self.__context_menu.append(item)
				
		self.__context_menu.popup(None,None,None,gdk_event_object.button,gdk_event_object.time)
 
		
	def __mouse_motion_handler(self, iconlist_obj, gdk_event_obj):
		icon_under_mouse = self.__iconlist.get_icon_at(int(gdk_event_obj.x), int(gdk_event_obj.y))
		
		if not icon_under_mouse == -1: 
			self.__active_profile = icon_under_mouse
			
		if not self.__start_double_click:
			if icon_under_mouse >= 0 and self.__cursor == "out":
				new_cursor = gtk.gdk.Cursor(gtk.gdk.HAND2)
				gdk_event_obj.window.set_cursor(new_cursor)
				self.__cursor = "over"
			elif icon_under_mouse == -1 and self.__cursor == "over":
				new_cursor = gtk.gdk.Cursor(gtk.gdk.LEFT_PTR)
				gdk_event_obj.window.set_cursor(new_cursor)
				self.__cursor = "out"

		
				
	def __get_icon_from_profile(self, profile_name):
		profile_icon = self.__gconf_client.get_string("/apps/gnome-terminal/profiles/"+profile_name+"/icon")

		if profile_icon[0] != "/":
			profile_icon = self.__icon_default_path + profile_icon

		return profile_icon


	def __starter(self):
		self.__start_with_window()

		
	def __start_with_window(self, menu_item = None):
		cmd = "gnome-terminal --window-with-profile=\""+gconf.unescape_key(self.__profile_list[self.__active_profile], len(self.__profile_list[self.__active_profile]))+"\""
		pid = gnome.execute_shell(".", cmd)
		if not self.__keep_open:
			gtk.main_quit()
		else:
			context_id = self.__statusbar.get_context_id("profile_started")
			cur_status_id = self.__statusbar.push(context_id, 'start terminal using profile "'+gconf.unescape_key(self.__profile_list[self.__active_profile], len(self.__profile_list[self.__active_profile])) + '"')

	"""

	def __change_profile_icon(self, menu_item = None):
		return
                fd = gtk.FileChooserDialog("Please choose an icon ...",
			None,
			gtk.FILE_CHOOSER_ACTION_OPEN,
			(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
			gtk.STOCK_OPEN, gtk.RESPONSE_OK))
                fd.set_icon(gtk.gdk.pixbuf_new_from_file(GTS_ICON))
		fd.set_local_only(gtk.TRUE)
		fd.set_current_folder("/usr/X11R6/share/gnome/pixmaps/")
		file_filter = gtk.FileFilter()
		file_filter.set_name("Images")
		file_filter.add_mime_type("image/png")
		file_filter.add_mime_type("image/jpeg")
		file_filter.add_mime_type("image/gif")
		file_filter.add_mime_type("image/svg+xml")
		fd.set_filter(file_filter)
		
		fd.set_preview_widget_active(gtk.TRUE)
                response = fd.run()
		
                if response == gtk.RESPONSE_OK:
                        selected = fd.get_filename();
                        self.__on_ok_fsel_dialog(splashdir_entry, selected)
			
		fd.destroy()

				
	def __set_statusbar_text(self, profile_nr):
		context_id = self.__statusbar.get_context_id("Mouseover Button")
		cur_status_id = self.__statusbar.push(context_id, gconf.unescape_key(self.__profile_list[profile_nr], len(self.__profile_list[profile_nr])))

	"""

	def __show_about(self, menu_nr, menu_item_obj):
		pixbuf = gtk.gdk.pixbuf_new_from_file(GTS_ABOUT_PIX)
		gnome_about = gnome.ui.About(SCRIPT_NAME, SCRIPT_VERSION, SCRIPT_COPYRIGHT,
						SCRIPT_DESCR, SCRIPT_AUTHORS, SCRIPT_DOCU, SCRIPT_TRANS, pixbuf)
		gnome_about.set_icon(gtk.gdk.pixbuf_new_from_file(GTS_ICON))
		gnome_about.show()

